@@ -11,6 +11,7 @@ tests_init \
 	libs_cflags_version_alt \
 	libs_cflags_version_different \
 	libs_cflags_version_different_bad \
+	libs_env \
 	exists_nonexitent \
 	nonexitent \
 	exists_version \
@@ -21,6 +22,8 @@ tests_init \
 	exists2 \
 	exists3 \
 	exists_version_alt \
+	exists_cflags \
+	exists_cflags_env \
 	uninstalled_bad \
 	uninstalled \
 	libs_intermediary \
@@ -39,7 +42,9 @@ tests_init \
 	arbitary_path \
 	with_path \
 	relocatable \
-	single_depth_selectors
+	single_depth_selectors \
+	print_variables_env \
+	variable_env
 
 noargs_body()
 {
@@ -340,3 +345,39 @@ modversion_noflatten_body()
 		-o inline:"1.3\n" \
 		pkgconf --with-path=${selfdir}/lib1 --modversion bar
 }
+
+exists_cflags_body()
+{
+	atf_check \
+		-o inline:"-DHAVE_FOO\n" \
+		pkgconf --with-path=${selfdir}/lib1 --cflags --exists-cflags --fragment-filter=D foo
+}
+
+exists_cflags_env_body()
+{
+	atf_check \
+		-o inline:"FOO_CFLAGS='-DHAVE_FOO'\n" \
+		pkgconf --with-path=${selfdir}/lib1 --cflags --exists-cflags --fragment-filter=D --env=FOO foo
+}
+
+libs_env_body()
+{
+	atf_check \
+		-o inline:"FOO_LIBS='-L/test/lib -lfoo'\n" \
+		pkgconf --with-path=${selfdir}/lib1 --libs --env=FOO foo
+}
+
+print_variables_env_body()
+{
+	atf_check \
+		-o inline:"FOO_CFLAGS='-fPIC -I/test/include/foo'\nFOO_LIBS='-L/test/lib -lfoo'\nFOO_INCLUDEDIR='/test/include'\nFOO_LIBDIR='/test/lib'\nFOO_EXEC_PREFIX='/test'\nFOO_PREFIX='/test'\nFOO_PCFILEDIR='${selfdir}/lib1'\n" \
+		pkgconf --with-path=${selfdir}/lib1 --env=FOO --print-variables --cflags --libs foo
+
+}
+
+variable_env_body()
+{
+	atf_check \
+		-o inline:"FOO_INCLUDEDIR='/test/include'\n" \
+		pkgconf --with-path=${selfdir}/lib1 --env=FOO --variable=includedir foo
+}
